import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.StringTokenizer;

import javafx.application.Application;
import javafx.scene.Group;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.CullFace;
import javafx.scene.shape.MeshView;
import javafx.scene.shape.TriangleMesh;
import javafx.scene.shape.VertexFormat;

/**
 * オブジェクトリーダー
 * <pre>
 * 3Dオブジェクトの元になるデータを読み込み、保持します。
 * 保持は、クラス名をキーに保持し、HashMapの形で内部に保持します。
 * createメソッドが呼ばれたタイミングで実際の3Dデータとして生成します。
 * メモリ使用量低減とスピードアップが目的です。
 *
 * 多角形に対応。
 * ただし、内角は180度を超えてはいけません。
 * 内角が180度を超えた場合、表示がおかしくなります。
 * </pre>
 */
public class ObjReader {

	/** ダミーテクスチャ部分 */
	private static final float[]				TEXCOORDS_DUMMY		= {
		0, 0
	};

	/** オブジェクト名（デフォルト） */
	public static final String					OBJECT_NAME_DEFAULT	= "";

	/** アプリケーション */
	protected static Application				application			= null;

	/** 頂点一覧（スプライトマップ） */
	private static HashMap<String, float[]>		mapPoints			= null;

	/** テクスチャ座標一覧（スプライトマップ） */
	private static HashMap<String, float[]>		mapTexs				= null;

	/** 頂点法線一覧（スプライトマップ） */
	private static HashMap<String, float[]>		mapNormals			= null;

	/** オブジェクト一覧（スプライトマップ） */
	private static HashMap<String, ArrayList<Object[]>>	mapObjects			= null;

	/**
	 * デバッグレベル
	 * <pre>
	 * 通常は0にしてください。
	 * １で概要レベル、２、３で詳細レベルまで出力します。
	 * </pre>
	 */
	private static int							lvlDebug			= 0;

	/**
	 * static 初期化
	 * <pre>
	 * obj／mtlファイルの情報を内部に保持するための初期化を行います。
	 * </pre>
	 */
	static {

		/** 頂点一覧（スプライトマップ） */
		mapPoints = new HashMap<String, float[]>();

		/** テクスチャ座標一覧（スプライトマップ） */
		mapTexs = new HashMap<String, float[]>();

		/** 頂点法線一覧（スプライトマップ） */
		mapNormals = new HashMap<String, float[]>();

		/** オブジェクト一覧（スプライトマップ） */
		mapObjects = new HashMap<String, ArrayList<Object[]>>();

	} // end static

	/**
	 * アプリケーション設定
	 * <pre>
	 * ファイルパスを取得するために、アプリケーションクラスを設定します。
	 * initメソッドを呼び出す前に、当設定を行ってください。
	 * </pre>
	 * @param application アプリケーション
	 */
	public static void setApplication(Application application){
		ObjReader.application = application;
	}

	/**
	 * 初期化
	 * <pre>
	 * 初期化を行う。
	 * ファイル名を指定し、3Dオブジェクトの元になるデータを保持します。
	 * 保持は、クラス名をキーに保持し、HashMapの形で内部に保持します。
	 * createメソッドが呼ばれたタイミングで実際の3Dデータとして生成します。
	 * 基本的には、当メソッドを利用してください。
	 * </pre>
	 * @param classSprite 登録するクラス。当クラスに読み込んだオブジェクトを配置します。
	 * @param strFilePath ファイルパス
	 */
	public static void init(Class<?> classSprite, String strFilePath) throws Exception {

		// スプライト登録キーを生成
		String strClassSprite = classSprite.getName();

		// 別メソッドを呼び出し
		init(strClassSprite, strFilePath);

	} // end init

	/**
	 * 初期化
	 * <pre>
	 * 初期化を行う。
	 * ファイル名を指定し、3Dオブジェクトの元になるデータを保持します。
	 * 保持は、Stringをキーに保持し、HashMapの形で内部に保持します。
	 * createメソッドが呼ばれたタイミングで実際の3Dデータとして生成します。
	 * </pre>
	 * @param strSpriteKey スプライトを登録するキー。読み込んだオブジェクトを当キーで保持します。
	 * @param strFilePath ファイルパス
	 */
	public static void init(String strSpriteKey, String strFilePath) throws Exception {

		// デフォルトパス
		String				strDefaultPath	= null;

		// InputStream
		InputStream			is				= null;

		// InputStreamReader
		InputStreamReader	isr				= null;

		// LineNumberReader
		LineNumberReader	lnr				= null;

		// 行（エラー表示用）
		int row = 0;

		try{

			// デバッグ情報出力
			outputDebugInfo(1, "START ObjReader.init. strFilePath:" + strFilePath);

			// ファイルパスのデフォルトパスを保持
			int indexDefaultPath = strFilePath.lastIndexOf("/");
			if(indexDefaultPath == -1){
				strDefaultPath = "";
			}else{
				strDefaultPath = strFilePath.substring(0, indexDefaultPath + 1);
			}

			// InputStreamを生成
			is = ObjReader.application.getClass().getResourceAsStream(strFilePath);

			// ファイルが見つからない場合、
			if(is == null){
				// エラーをスロー
				throw new IllegalStateException("File Not Found. filePath = " + strFilePath);
			}

			// InputStreamReaderを生成
			isr = new InputStreamReader(is);

			// LineNumberReaderを生成
			lnr = new LineNumberReader(isr);

			// 頂点座標一覧を生成
			ArrayList<Float> pointLst = new ArrayList<Float>();
			// テクスチャ一覧を生成
			ArrayList<Float> texLst = new ArrayList<Float>();
			// 法線一覧を生成
			ArrayList<Float> normalLst = new ArrayList<Float>();

			// オブジェクト一覧を生成
			ArrayList<Object[]> objectsLst = new ArrayList<Object[]>();

			// 面要素一覧を生成
			ArrayList<Integer> faceLst = new ArrayList<Integer>();
			// マテリアル一覧を生成
			ArrayList<Object[]> materialLst = new ArrayList<Object[]>();

			// 面要素数
			int countFaces = 2;

			// オブジェクト名にデフォルト値を設定
			String strObjectName = OBJECT_NAME_DEFAULT;

			// マテリアル名
			String strMaterialName = null;

			// マテリアルマップ
			HashMap<String, PhongMaterial> mapMaterial = null;

			// 行数分、ループ
			while(true){

				// 行＋１
				row++;

				// １行取得
				String strLine = lnr.readLine();

				// 行情報が取得できない場合、
				if(strLine == null){
					// ループを抜ける
					break;
				}

				// トークンを生成
				StringTokenizer stLine = new StringTokenizer(strLine, " ");

				// 文字列一覧生成
				ArrayList<String> strLst = new ArrayList<String>(5);

				// トークン分、ループ
				while(true){

					// トークンがない場合、
					if(!stLine.hasMoreTokens()){
						// ループを抜ける
						break;
					}

					// トークンを文字列一覧に格納
					strLst.add(stLine.nextToken());

				} // end while トークン分、ループ

				// 文字列が取得できなかった場合、
				if(strLst.size() == 0){
					// コンティニュー
					continue;
				}

				//
				// 各情報を取得
				//

				// 頂点の場合、
				if(strLst.get(0).equals("v")){

					// 頂点を取得
					Float floatX = Float.valueOf((String)strLst.get(1));
					Float floatY = Float.valueOf((String)strLst.get(2));
					Float floatZ = Float.valueOf((String)strLst.get(3));

					// 頂点一覧に追加
					pointLst.add(floatX);
					pointLst.add(-floatY);
					pointLst.add(-floatZ);

				// テクスチャの場合、
				}else if(strLst.get(0).equals("vt")){

					// テクスチャを取得
					Float floatU = Float.valueOf((String)strLst.get(1));
					Float floatV = -Float.valueOf((String)strLst.get(2));

					// テクスチャ一覧に追加
					texLst.add(floatU);
					texLst.add(floatV);

				// 法線の場合、
				}else if(strLst.get(0).equals("vn")){

					// 法線を取得
					Float floatX = Float.valueOf((String)strLst.get(1));
					Float floatY = Float.valueOf((String)strLst.get(2));
					Float floatZ = Float.valueOf((String)strLst.get(3));

					// 法線一覧に追加
					normalLst.add(floatX);
					normalLst.add(-floatY);
					normalLst.add(-floatZ);

				// 面要素の場合、
				}else if(strLst.get(0).equals("f")){

					// 行情報を使いやすいように、０番目の要素を削除
					strLst.remove(0);

					// 何角形？
					int intPolygon = strLst.size();

					// ３角形が存在する場合、
					while(intPolygon >= 3){

						//
						// １点目
						//
						String str1 = strLst.get(0);

						// １点目情報分解
						int[] intFaces1 = getFaces(str1);

						// 法線情報ありの場合、
						// （スラッシュが２つある場合）
						if(intFaces1.length > countFaces){
							// 面要素数
							countFaces = intFaces1.length;
						}

						// 頂点
						int v1 = intFaces1[0];
						faceLst.add(v1);

						int vn1 = 0;

						// 法線
						if(countFaces == 3){
							vn1 = intFaces1[2];
							faceLst.add(vn1);
						}

						// テクスチャ
						int vt1 = intFaces1[1];
						faceLst.add(vt1);

						//
						// ２点目
						//
						String str2 = strLst.get(1);

						// ２点目情報分解
						int[] intFaces2 = getFaces(str2);

						// 頂点
						int v2 = intFaces2[0];
						faceLst.add(v2);

						int vn2 = 0;

						// 法線
						if(countFaces == 3){
							vn2 = intFaces2[2];
							faceLst.add(vn2);
						}

						// テクスチャ
						int vt2 = intFaces2[1];
						faceLst.add(vt2);

						//
						// ３点目
						//
						String str3 = strLst.get(2);

						// ３点目情報分解
						int[] intFaces3 = getFaces(str3);

						// 頂点
						int v3 = intFaces3[0];
						faceLst.add(v3);

						int vn3 = 0;

						// 法線
						if(countFaces == 3){
							vn3 = intFaces3[2];
							faceLst.add(vn3);
						}

						// テクスチャ
						int vt3 = intFaces3[1];
						faceLst.add(vt3);

						// １角形小さくする
						intPolygon--;

						// １番目（２点目）の要素を削除する
						strLst.remove(1);

					} // end while ３角形が存在する場合

				// マテリアルの場合、
				}else if(strLst.get(0).equals("usemtl")){

					// マテリアル追加／チェック
					Object[] materials = ObjReader.addCheckMaterial(faceLst, mapMaterial, strMaterialName);

					// マテリアルが返却されてきた場合、
					if(materials != null){

						// マテリアル一覧に追加
						materialLst.add(materials);

						// 面要素一覧を生成（初期化）
						// 次の面要素一覧を準備
						// （忘れないようにメモ）
						// ここにnewを入れるのに、半月以上かかった・・・。
						// マテリアルが最後のものが適用されたように見えてしまう。
						faceLst = new ArrayList<Integer>();

					}

					// マテリアル名を取得
					strMaterialName = strLst.get(1);

				// オブジェクトの場合、
				}else if(strLst.get(0).equals("o")){

					// マテリアル追加／チェック
					Object[] materials = ObjReader.addCheckMaterial(faceLst, mapMaterial, strMaterialName);

					// マテリアルが返却されてきた場合、
					if(materials != null){

						// マテリアル一覧に追加
						materialLst.add(materials);

						// 面要素一覧を生成（初期化）
						// 次の面要素一覧を準備
						// （忘れないようにメモ）
						// ここにnewを入れるのに、半月以上かかった・・・。
						// マテリアルが最後のものが適用されたように見えてしまう。
						faceLst = new ArrayList<Integer>();

					}

					// マテリアル名を初期化
					// マテリアル名を一旦、初期化
					strMaterialName = null;

					// オブジェクト追加／チェック
					Object[] objects = ObjReader.addObject(strObjectName, materialLst);

					// オブジェクトが返却されてきた場合、
					if(materials != null){

						// オブジェクト一覧に追加
						objectsLst.add(objects);

					}

					// マテリアル一覧を初期化
					// オブジェクト「o」が変わっているので、マテリアル一覧の初期化が必要
					materialLst = new ArrayList<Object[]>();

					// オブジェクト名を設定
					strObjectName = strLst.get(1);

				// マテリアル参照の場合、
				}else if(strLst.get(0).equals("mtllib")){

					// マテリアルリーダーよりマテリアルマップ取得
					mapMaterial = MaterialReader.getMaterials(strDefaultPath + strLst.get(1));

				} // end if

			} // end while 行数分、ループ

			// マテリアル追加／チェック
			Object[] materials = ObjReader.addCheckMaterial(faceLst, mapMaterial, strMaterialName);

			// マテリアルが返却されてきた場合、
			if(materials != null){

				// マテリアル一覧に追加
				materialLst.add(materials);

			}

			// オブジェクト追加／チェック
			Object[] objects = ObjReader.addObject(strObjectName, materialLst);

			// オブジェクトが返却されてきた場合、
			if(materials != null){

				// オブジェクト一覧に追加
				objectsLst.add(objects);

			}

			// オブジェクト一覧（スプライトマップ）に追加
			ObjReader.mapObjects.put(strSpriteKey, objectsLst);

			// 頂点一覧（スプライトマップ）に追加
			ObjReader.mapPoints.put(strSpriteKey, getFloats(pointLst));

			// テクスチャ座標一覧（スプライトマップ）に追加
			ObjReader.mapTexs.put(strSpriteKey, getFloats(texLst));

			// 頂点法線一覧（スプライトマップ）に追加
			ObjReader.mapNormals.put(strSpriteKey, getFloats(normalLst));

			// デバッグ情報出力
			outputDebugInfo(1, "END ObjReader.init. strFilePath:" + strFilePath);

			// リターン
			return;

		}catch(Exception ex){

			// 行が途中の場合、
			if(row > 0){
				// エラーのあった行を出力
				System.out.println("Error! row = : " + row);
			}

			// そのままエラーをスロー
			throw ex;

		}finally{

			try{
				// クローズ
				if(isr != null){
					isr.close();
				}
			}catch(Exception ex){
				ex.printStackTrace();
			}

		}

	} // end init

	/**
	 * オブジェクト追加／チェック
	 */
	private static Object[] addObject(String strObjectName, ArrayList<Object[]> materialLst){

		// マテリアル一覧が存在する場合、
		if(materialLst.size() > 0){

			// 返却用オブジェクトを作成
			Object[] objObjects = new Object[2];

			// 返却用オブジェクトを設定
			objObjects[0] = strObjectName;
			objObjects[1] = materialLst;

			// リターン
			return objObjects;

		// マテリアル一覧が存在しない場合、
		}else{

			// nullでリターン
			return null;

		}

	} // end addObject

	/**
	 * マテリアル追加／チェック
	 */
	private static Object[] addCheckMaterial(ArrayList<Integer> faceLst, HashMap<String, PhongMaterial> mapMaterial, String strMaterialName){

		// 面要素が存在する場合、
		// メッシュビューが追加できる場合、
		if(faceLst.size() > 0){

			// マテリアルを作成
			Object[] materials = createMaterial(faceLst, mapMaterial, strMaterialName);

			// リターン
			return materials;

		// 面要素が存在しない場合、
		}else{

			// nullでリターン
			return null;

		}

	} // end addCheckMaterial

	/**
	 * マテリアル作成
	 */
	private static Object[] createMaterial(ArrayList<Integer> faceLst, HashMap<String, PhongMaterial> mapMaterial, String strMaterialName){

		// デバッグ情報出力
		outputDebugInfo(2, "START createMaterial.");

		// 返却用マテリアルを作成
		Object[] objMaterials = new Object[2];

		//
		// 面要素一覧
		//

		// int[]で取得
		int[] intFaces = getInts(faceLst);

		// デバッグ情報出力
		outputDebugInfo(2, "intFaces：" + intFaces.length);

		//
		// マテリアル
		//

		// マテリアル
		PhongMaterial material = null;

		// マテリアル名が保持されている場合、
		if(strMaterialName != null){

			// マテリアルを取得
			material = mapMaterial.get(strMaterialName);

			// デバッグ情報出力
			outputDebugInfo(2, "マテリアル適用 usemtl：" + strMaterialName);

		} // end if マテリアル名が保持されている場合

		// 返却用マテリアルを設定
		objMaterials[0] = intFaces;
		objMaterials[1] = material;

		// デバッグ情報出力
		outputDebugInfo(2, "END createMaterial.");

		// 返却用マテリアルをリターン
		return objMaterials;

	} // end createMaterial

	/**
	 * 面要素取得
	 */
	private static int[] getFaces(String strFace){

		// インデックスを取得
		int index_vt = -1;
		int index_tn = -1;

		// インデックス頂点−テクスチャを取得
		index_vt = strFace.indexOf("/");

		int v = -1;
		int vt = -1;
		int vn = -1;

		// 頂点を取得
		if(index_vt != -1){
			v = Integer.valueOf(strFace.substring(0, index_vt)) - 1;
		}else{
			v = Integer.valueOf(strFace) - 1;
		}

		// インデックス頂点−テクスチャが取得できた場合、
		if(index_vt != -1){

			// インデックステクスチャを取得
			index_tn = strFace.indexOf("/", index_vt + 1);

			// テクスチャを取得
			if(index_vt + 1 == index_tn){
				vt = 0;
			}else if(index_tn != -1){
				vt = Integer.valueOf(strFace.substring(index_vt + 1, index_tn)) - 1;
			}else{
				vt = Integer.valueOf(strFace.substring(index_vt + 1)) - 1;
			}

			// インデックステクスチャが取得できた場合、
			if(index_tn != -1){

				// 法線を取得
				if(index_tn + 1 == strFace.length()){
					vn = 0;
				}else{
					vn = Integer.valueOf(strFace.substring(index_tn + 1)) - 1;
				}

			}

		// インデックス頂点−テクスチャが取得できなかった場合、
		}else{
			vt = 0;
		}

		// デバッグ情報出力
		outputDebugInfo(3, "v :" + v + " / vt:" + vt + " / vn:" + vn);

		int[] intFaces = null;

		if(vn == -1){
			intFaces = new int[2];
			intFaces[0] = v;
			intFaces[1] = vt;
		}else{
			intFaces = new int[3];
			intFaces[0] = v;
			intFaces[1] = vt;
			intFaces[2] = vn;
		}

		return intFaces;

	} // end getFaces

	/**
	 * 3Dオブジェクト生成
	 * <pre>
	 * 基本的には、当メソッドを利用してください。
	 * </pre>
	 * @param group 登録するグループ（スプライト単位）。当グループ（クラス−オブジェクト）に、生成したオブジェクトを配置します。
	 * @return HashMap オブジェクト「o」単位のGroupオブジェクト
	 */
	public static HashMap<String, Group> create(Group group){

		// スプライトを取得するキー
		String strSpriteKey = group.getClass().getName();

		// 別メソッドを呼び出し
		return create(group, strSpriteKey);

	} // end create

	/**
	 * 3Dオブジェクト生成
	 * @param group 登録するグループ（スプライト単位）。当グループに、生成したオブジェクトを配置します。
	 * @param strSpriteKey スプライトを取得するキー
	 * @return HashMap オブジェクト「o」単位のGroupオブジェクト
	 */
	public static HashMap<String, Group> create(Group group, String strSpriteKey){

		// デバッグ情報出力
		outputDebugInfo(1, "START ObjReader.create. " + group);

		// オブジェクト一覧（スプライト単位）を取得
		ArrayList<Object[]> objectsLst = ObjReader.mapObjects.get(strSpriteKey);

		// 頂点一覧（スプライト単位）を取得
		float[] floatPoints = ObjReader.mapPoints.get(strSpriteKey);

		// テクスチャ座標一覧（スプライト単位）を取得
		float[] floatTexs = ObjReader.mapTexs.get(strSpriteKey);

		// 頂点法線一覧（スプライト単位）を取得
		float[] floatNormals = ObjReader.mapNormals.get(strSpriteKey);

		// オブジェクト一覧の件数を取得
		int countObjectsLst = objectsLst.size();

		// デバッグ情報出力
		outputDebugInfo(2, "スプライトデータ取得ＯＫ？ " + ((countObjectsLst > 0)?"ＯＫ！": "ＮＧ！"));

		// グループマップ（オブジェクト単位）を生成
		HashMap<String, Group> mapGroup = new HashMap<String, Group>(countObjectsLst);

		// オブジェクト一覧の件数分、ループ
		for(int i = 0; i < countObjectsLst; i++){

			// オブジェクトを取得
			Object[] objects = objectsLst.get(i);

			// オブジェクト名を取得
			String strObjectName = (String)objects[0];

			// グループ（オブジェクト単位）を生成
			Group groupObject = new Group();

			// グループ（スプライト単位）に追加
			group.getChildren().add(groupObject);

			// グループマップ（オブジェクト単位）に追加
			mapGroup.put(strObjectName, groupObject);

			// マテリアル一覧を取得
			Object o = objects[1];
			@SuppressWarnings("unchecked")
			ArrayList<Object[]> materialLst = (ArrayList<Object[]>)o;

			// マテリアル一覧の件数を取得
			int countMaterialLst = materialLst.size();

			// デバッグ情報出力
			outputDebugInfo(2, "マテリアル一覧取得ＯＫ？ " + ((countMaterialLst > 0)?"ＯＫ！": "ＮＧ！"));

			// マテリアル一覧の件数分、ループ
			for(int j = 0; j < countMaterialLst; j++){

				// マテリアルを取得
				Object[] objMaterials = (Object[])materialLst.get(j);

				// 面要素一覧取得
				int[] intFaces = (int[])objMaterials[0];

				// マテリアルを取得
				PhongMaterial material = (PhongMaterial)objMaterials[1];

				// メッシュビューを作成
				MeshView meshView = new MeshView();

				// メッシュを作成
				TriangleMesh mesh = new TriangleMesh();

				//
				// テックス／頂点法線一覧設定
				//

				// 頂点法線が存在する場合、
				if(floatNormals != null && floatNormals.length > 0){

					// テックスはノーマル
					mesh.setVertexFormat(VertexFormat.POINT_NORMAL_TEXCOORD);

					// 頂点法線一覧を設定
					mesh.getNormals().addAll(floatNormals);

				// 上記以外の場合、
				}else{

					// テックスはノーマルではない
					mesh.setVertexFormat(VertexFormat.POINT_TEXCOORD);

					// 両面を表示するようにします。
					// JavaFXでは、反時計回りに点が見える場合に、表側と判断している。
					// objファイルも法線がない場合でも反時計回りが表向きになるようだが、
					// 両面とも、表示するようにします。
					meshView.setCullFace(CullFace.NONE);

				}

				//
				// 頂点座標一覧
				//

				// 頂点座標一覧を設定
				mesh.getPoints().addAll(floatPoints);

				//
				// テクスチャ座標一覧
				//

				// テクスチャが設定された場合、
				if(floatTexs != null && floatTexs.length > 0){

					// テクスチャ一覧を設定
					mesh.getTexCoords().addAll(floatTexs);

				// テクスチャが設定されなかった場合、
				}else{

					// テクスチャのダミーを設定
					mesh.getTexCoords().addAll(TEXCOORDS_DUMMY);

				}

				//
				// 面要素一覧
				//

				// 面要素一覧を設定
				mesh.getFaces().addAll(intFaces);

				//
				// メッシュビュー登録
				//

				// メッシュを登録
				meshView.setMesh(mesh);

				// マテリアルを取得できた場合、
				if(material != null){

					// マテリアルを登録
					meshView.setMaterial(copyMaterial(material));

					// デバッグ情報出力
					outputDebugInfo(2, "マテリアル登録 " + material);

				}

				// グループ（オブジェクト単位）にメッシュビューを追加
				groupObject.getChildren().add(meshView);

			} // end for マテリアル一覧の件数分、ループ

		} // end for オブジェクト一覧の件数分、ループ

		// デバッグ情報出力
		outputDebugInfo(1, "END ObjReader.create. " + group);

		// グループマップ（オブジェクト単位）をリターン
		return mapGroup;

	} // end create

	/**
	 * マテリアルコピー
	 */
	private static PhongMaterial copyMaterial(PhongMaterial material){

		// マテリアルを生成
		PhongMaterial materialWk = new PhongMaterial();

		// 各種情報をコピー
		materialWk.setBumpMap(material.getBumpMap());
		materialWk.setDiffuseColor(material.getDiffuseColor());
		materialWk.setDiffuseMap(material.getDiffuseMap());
		materialWk.setSelfIlluminationMap(material.getSelfIlluminationMap());
		materialWk.setSpecularColor(material.getSpecularColor());
		materialWk.setSpecularMap(material.getSpecularMap());
		materialWk.setSpecularPower(material.getSpecularPower());

		// マテリアルをリターン
		return materialWk;

	} // end copyMaterial

	/**
	 * float値取得
	 * @param floatLst ArrayList<Float>
	 * @return float[]
	 */
	private static float[] getFloats(ArrayList<Float> floatLst){

		// 一覧の件数を取得
		int countFloatLst = floatLst.size();

		// 一覧の件数の要素数で生成
		float[] floats = new float[countFloatLst];

		// データの転送を行う
		for(int i = 0; i < countFloatLst; i++){

			floats[i] = floatLst.get(i).floatValue();

		}

		// リターン
		return floats;

	} // end getFloats

	/**
	 * int値取得
	 * @param floatLst ArrayList<Integer>
	 * @return int[]
	 */
	private static int[] getInts(ArrayList<Integer> intLst){

		// 一覧の件数を取得
		int countIntLst = intLst.size();

		// 一覧の件数の要素数で生成
		int[] ints = new int[countIntLst];

		// データの転送を行う
		for(int i = 0; i < countIntLst; i++){

			ints[i] = intLst.get(i).intValue();

		}

		// リターン
		return ints;

	} // end getInts

	/**
	 * デバッグ情報出力
	 * <pre>
	 * デバッグレベルをチェックし、コンソールに情報を出力します。
	 * </pre>
	 * @param strDebugInfo デバッグ情報
	 */
	private static void outputDebugInfo(int lvlDebug, String strDebugInfo){

		if(lvlDebug <= ObjReader.lvlDebug){

			System.out.println(strDebugInfo);

		}

	} // end outputDebugInfo

}
